<!---
	COMPONENT: ContactDAO
	AUTHOR: Matt Woodward (mpwoodward@gmail.com)
	DATE: 7/17/2005
	PURPOSE: ContactDAO for Mach-II Contact Manager sample application
	CHANGE HISTORY:
		* 2/28/2006: component created (Matt Woodward)
--->

<cfcomponent displayname="ContactDAO" output="false" hint="Contact DAO for Mach-II Contact Manager sample application">
	<cffunction name="init" access="public" output="false" returntype="ContactDAO" hint="Constructor for this CFC">
		<!--- take in the datasource name as an argument and set it to the variables scope so it's available 
				throughout the CFC --->
		<cfargument name="dsn" type="string" required="true" />
		<cfset variables.dsn = arguments.dsn />
		
		<!--- return the object itself --->
		<cfreturn this />
	</cffunction>
	
	<!--- The classic four methods included in a Data Access Object (DAO) are the so-called 'CRUD' methods, 
			which stands for Create, Read, Update, and Delete.  Traditionally each of these four methods 
			will take in a bean of the type managed by the specific DAO (in this case that means a Contact 
			bean) and perform the respective tasks.  Since objects are passed by reference you'll notice that 
			the read method doesn't return an object, but rather the operations are performed on the original 
			object itself.  The other methods return a struct containing a boolean indicating the success or 
			failure of the event as well as a message string.  This is the way I usually do things but 
			this is not mandatory. --->
	<!--- CREATE: creates a new record in the database using the bean passed in --->
	<cffunction name="create" access="public" output="false" returntype="struct" 
			hint="Creates a new record and returns a struct containing a boolean and a string">
		<cfargument name="contact" type="Contact" required="true" />
		
		<!--- var scope everything! --->
		<cfset var insertContact = "" />
		<cfset var results = StructNew() />
		<cfset results.success = true />
		<cfset results.message = "The contact was created successfully." />
		
		<cftry>
			<cfquery name="insertContact" datasource="#variables.dsn#">
				INSERT INTO contact (
					id, 
					first_name, 
					last_name, 
					street, 
					city,
					state,
					zip,
					dt_created
				) VALUES (
					<cfqueryparam value="#arguments.contact.getId()#" cfsqltype="cf_sql_char" />, 
					<cfqueryparam value="#arguments.contact.getFirstName()#" cfsqltype="cf_sql_varchar" />, 
					<cfqueryparam value="#arguments.contact.getLastName()#" cfsqltype="cf_sql_varchar" />, 
					<cfqueryparam value="#arguments.contact.getStreet()#" cfsqltype="cf_sql_varchar" />, 
					<cfqueryparam value="#arguments.contact.getCity()#" cfsqltype="cf_sql_varchar" />, 
					<cfqueryparam value="#arguments.contact.getState()#" cfsqltype="cf_sql_varchar" />, 
					<cfqueryparam value="#arguments.contact.getZip()#" cfsqltype="cf_sql_varchar" />, 
					<cfqueryparam value="#Now()#" cfsqltype="cf_sql_timestamp" />
				)
			</cfquery>
			<cfcatch type="database">
				<cfset results.success = false />
				<cfset results.message = "A database error occurred: #CFCATCH.Detail#" />
			</cfcatch>
		</cftry>
		
		<cfreturn results />
	</cffunction>
	
	<!--- READ: populates a contact bean using info from the database --->
	<cffunction name="read" access="public" output="false" returntype="void" 
			hint="Populates the contact bean using the ID of the object passed in">
		<cfargument name="contact" type="Contact" required="true" />
		
		<!--- var scope everything! --->
		<cfset var getContact = "" />
		
		<cfquery name="getContact" datasource="#variables.dsn#">
			SELECT * 
			FROM contact 
			WHERE id = <cfqueryparam value="#arguments.contact.getId()#" cfsqltype="cf_sql_char" />
		</cfquery>
		
		<!--- if we got a record back, populate the bean --->
		<cfif getContact.RecordCount EQ 1>
			<cfset arguments.contact.init(getContact.id, getContact.first_name, getContact.last_name, 
											getContact.street, getContact.city, getContact.state, 
											getContact.zip) />
		</cfif>
	</cffunction>
	
	<!--- UPDATE: updates an existing record in the database using the data in the bean that's passed in --->
	<cffunction name="update" access="public" output="false" returntype="struct" 
			hint="Updates an existing record in the database and returns a struct containing a boolean and a string">
		<cfargument name="contact" type="Contact" required="true" />
		
		<!--- yes, you still need to VAR SCOPE EVERYTHING!  Simpsons fans, if you saw the recent episode where the gym 
				teacher kept yelling "BOMBARDMENT" and pegging the kids with a dodge ball, that's the same thing we're 
				getting into here.  Picture me yelling "VAR SCOPE!" and hurling a dodge ball at you. --->
		<cfset var updateContact = "" />
		<cfset var results = StructNew() />
		<cfset results.success = true />
		<cfset results.message = "The contact was updated successfully." />
		
		<cftry>
			<cfquery name="updateContact" datasource="#variables.dsn#">
				UPDATE 	contact 
				SET 	first_name = <cfqueryparam value="#arguments.contact.getFirstName()#" cfsqltype="cf_sql_varchar" />, 
						last_name = <cfqueryparam value="#arguments.contact.getLastName()#" cfsqltype="cf_sql_varchar" />, 
						street = <cfqueryparam value="#arguments.contact.getStreet()#" cfsqltype="cf_sql_varchar" />, 
						city = <cfqueryparam value="#arguments.contact.getCity()#" cfsqltype="cf_sql_varchar" />, 
						state = <cfqueryparam value="#arguments.contact.getState()#" cfsqltype="cf_sql_varchar" />, 
						zip = <cfqueryparam value="#arguments.contact.getZip()#" cfsqltype="cf_sql_varchar" /> 
				WHERE 	id = <cfqueryparam value="#arguments.contact.getId()#" cfsqltype="cf_sql_char" />
			</cfquery>
			<cfcatch type="database">
				<cfset results.success = false />
				<cfset results.message = "A database error occurred: #CFCATCH.Detail#" />
			</cfcatch>
		</cftry>
		
		<cfreturn results />
	</cffunction>
	
	<!--- DELETE: deletes a record --->
	<cffunction name="delete" access="public" output="false" returntype="struct" 
			hint="Deletes a record and returns a struct containing a boolean and a message">
		<cfargument name="contact" type="Contact" required="true" />
		
		<!--- VAR SCOPE!!! OK, strictly speaking you don't have to var scope things that don't return anything, 
				but just get in the habit of doing it for everything instead of figuring out what you do and don't 
				need to var scope.  Do it or the next dodge ball is aimed square at your noggin. --->
		<cfset var deleteContact = "" />
		<cfset var results = StructNew() />
		<cfset results.success = true />
		<cfset results.message = "The contact was deleted successfully." />
		
		<cftry>
			<cfquery name="deleteContact" datasource="#variables.dsn#">
				DELETE FROM contact 
				WHERE id = <cfqueryparam value="#arguments.contact.getId()#" cfsqltype="cf_sql_char" />
			</cfquery>
			<cfcatch type="database">
				<cfset results.success = false />
				<cfset results.message = "A database error occurred: #CFCATCH.Detail#" />
			</cfcatch>
		</cftry>
		
		<cfreturn results />
	</cffunction>
</cfcomponent>
