<!---
	COMPONENT: ContactListner
	AUTHOR: Matt Woodward (mpwoodward@gmail.com)
	DATE: 7/17/2005
	PURPOSE: ContactListener for Mach-II Contact Manager sample application
				* based on ContactManager CFC from original Mach-II Contact Manager sample application
				* combined functionality from ContactManager and RecentContacts CFCs in original application 
					into a single CFC for the updated version of the application
	CHANGE HISTORY:
		* 7/17/2005: component created (Matt Woodward)
		* 2/28/2006: updated for database version of the sample app
--->

<cfcomponent displayname="ContactListener" output="false" extends="MachII.framework.Listener" 
		hint="ContactListener for the Mach-II Contact Manager sample application">
	<!--- this configure method is called by Mach-II automatically when the application loads --->
	<cffunction name="configure" access="public" output="false" returntype="void" 
			hint="Configures this listener as part of the Mach-II framework">
		<!--- We'll need access to our dataobjects (gateway and DAO) in this Listener.  We get the datasource 
				name from the Mach-II properties we declared in the XML config file by using the getProperty function. --->
		<cfset variables.contactGateway = CreateObject("component", "ContactGateway").init(getProperty("dsn")) />
		<cfset variables.contactDAO = CreateObject("component", "ContactDAO").init(getProperty("dsn")) />
	</cffunction>
	
	<!--- GATEWAY-RELATED METHODS (gateways deal with *multiple* records) --->
	<!--- get all contacts - returns a query object containing all the contacts --->
	<cffunction name="getAllContacts" access="public" output="false" returntype="query" 
			hint="Returns a query object containing all the contacts">
		<cfreturn variables.contactGateway.getAllContacts() />
	</cffunction>
	
	<!--- get "recent" contacts, which we'll define as the last three added --->
	<cffunction name="getRecentContacts" access="public" output="false" returntype="query" 
			hint="Returns an query object containing recent contacts">
		<cfreturn variables.contactGateway.getRecentContacts() />
	</cffunction>

	<!--- DAO-RELATED METHODS (DAOs deal with *single* records) --->
	<!--- get contact - returns a specific contact object --->
	<cffunction name="getContact" access="public" output="false" returntype="Contact" 
			hint="Returns a specific contact object based on the id passed to this function">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<!--- Create an empty contact object.  This step is unnecessary in the event bean version 
				of this process (below). --->
		<cfset var contact = CreateObject("component", "Contact").init() />
		
		<!--- set the id from the event to the contact object --->
		<cfset contact.setId(arguments.event.getArg("id")) />
		
		<!--- pass call the read method on the DAO and pass it the contact bean with the id set --->
		<cfset variables.contactDAO.read(contact) />
		
		<!--- return the contact bean --->
		<cfreturn contact />
	</cffunction>
	
	<!--- create contact using simple form values as event arguments --->
	<cffunction name="createContactFromEventArgs" access="public" output="false" returntype="void" 
			hint="Creates a contact using the form values directly and announces the next event">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<!--- declare a struct that will hold the results from the call to the DAO to create a new 
				record --->
		<cfset var results = StructNew() />

		<!--- Instantiate contact bean and put form data from event object in it.
				This step is automated in the event bean version of this (below). --->
		<cfset var contact = CreateObject("component", 
											"Contact").init(CreateUUID(), 
															arguments.event.getArg("firstName"), 
															arguments.event.getArg("lastName"), 
															arguments.event.getArg("street"), 
															arguments.event.getArg("city"), 
															arguments.event.getArg("state"), 
															arguments.event.getArg("zip")) />
		
		<!--- set default results --->
		<cfset results.success = false />
		<cfset results.message = "An error occurred while attempting to create the contact." />
		
		
		<!--- Pass the contact bean to the DAO, which returns a struct with a boolean that indicates 
				whether or not the record was successfully created, and a message string. --->
		<cfset results = variables.contactDAO.create(contact) />
		
		<!--- put the message from the results struct in the event object so we can display it
				to the user at the end of the next event --->
		<cfset arguments.event.setArg("message", results.message) />
		
		<!--- announce the next event --->
		<cfset announceEvent("contactCreated", arguments.event.getArgs()) />
	</cffunction>
	
	<!--- create contact using event bean --->
	<cffunction name="createContactFromEventBean" access="public" output="false" returntype="void" 
			hint="Creates a contact using the Contact object as an event bean">
		<cfargument name="event" type="MachII.framework.Event" required="true" />

		<!--- declare a struct that will hold the results from the call to the DAO to create a new 
				record, and set default values for the struct's contents --->
		<cfset var results = StructNew() />
		<cfset results.success = false />
		<cfset results.message = "An error occurred while attempting to create the contact." />
		
		<!--- In this case since we're using an event bean, we already have a contact object in the 
				event object, and this contact object has been populated with the form data.  All we 
				need to do in this case is set a new ID in the contact bean and then pass it to the 
				DAO's create method. --->
		<cfset arguments.event.getArg("contact").setId(CreateUUID()) />
		
		<!--- pass the contact bean to the DAO's create method --->
		<cfset results = variables.contactDAO.create(arguments.event.getArg("contact")) />
		
		<!--- put the message from the results struct in the event object so we can display it
				to the user at the end of the next event --->
		<cfset arguments.event.setArg("message", results.message) />
		
		<!--- announce the next event --->
		<cfset announceEvent("contactCreated", arguments.event.getArgs()) />
	</cffunction>
	
	<!--- update contact using simple form values as event arguments (not used in app; uses event bean
		  	for updates by default) --->
	<cffunction name="updateContactFromEventArgs" access="public" output="false" returntype="void" 
			hint="Updates a contact using the form values directly and announces the next event">
		<cfargument name="event" type="MachII.framework.Event" required="true" />
		
		<!--- declare a struct that will hold the results from the call to the DAO to update the 
				record --->
		<cfset var results = StructNew() />
		
		<!--- Put the form data contained in the event object into a contact bean.  This step 
				isn't necessary when using an event bean (below). --->
		<cfset var contact = CreateObject("component", 
											"Contact").init(arguments.event.getArg("id"), 
															arguments.event.getArg("firstName"), 
															arguments.event.getArg("lastName"), 
															arguments.event.getArg("street"), 
															arguments.event.getArg("city"), 
															arguments.event.getArg("state"), 
															arguments.event.getArg("zip")) />
		
		<!--- set default results --->
		<cfset results.success = false />
		<cfset results.message = "An error occurred while attempting to update the contact." />
		
		<!--- pass the contact bean to the DAO's update method --->
		<cfset results = variables.contactDAO.update(contact) />
		
		<!--- put the message from the results struct in the event object so we can display it
				to the user at the end of the next event --->
		<cfset arguments.event.setArg("message", results.message) />
		
		<!--- announce the next event --->
		<cfset announceEvent("contactUpdated", arguments.event.getArgs()) />
	</cffunction>
	
	<!--- update contact using event bean --->
	<cffunction name="updateContactFromEventBean" access="public" output="false" returntype="void" 
			hint="Updates a contact using the Contact object as an event bean">
		<cfargument name="event" type="MachII.framework.Event" required="true" />

		<!--- declare a struct that will hold the results from the call to the DAO to update the 
				record, and set default values for the struct's contents --->
		<cfset var results = StructNew() />
		<cfset results.success = false />
		<cfset results.message = "An error occurred while attempting to update the contact." />
		
		<!--- In this case we have the contact bean already populated with the form data 
				because we used an event bean, so just pass it to the DAO's update method --->
		<cfset results = variables.contactDAO.update(arguments.event.getArg("contact")) />

		<!--- put the message from the results struct in the event object so we can display it
				to the user at the end of the next event --->
		<cfset arguments.event.setArg("message", results.message) />
		
		<!--- announce the next event --->
		<cfset announceEvent("contactUpdated", arguments.event.getArgs()) />
	</cffunction>
	
	<!--- delete contact --->
	<cffunction name="deleteContact" access="public" output="false" returntype="void" 
			hint="Deletes a contact and announces the next event">
		<cfargument name="event" type="MachII.framework.event" required="true" />
		
		<!--- declare a struct that will hold the results from the call to the DAO to update the 
				record, and set default values for the struct's contents --->
		<cfset var results = StructNew() />

		<!--- Even in the case of a delete, where all that's really needed is an id, by convention 
				the DAO will take in the entire object it manages as an argument to each of its 
				methods.  Since we aren't using an event bean here, we need to set the id from the 
				event to a contact bean and then pass that bean to the DAO's delete method.  The 
				rest of the data in the bean isn't necessary to do the delete. --->
		<cfset var contact = CreateObject("component", "Contact").init(arguments.event.getArg("id")) />
		
		<!--- set default results --->
		<cfset results.success = false />
		<cfset results.message = "An error occurred while attempting to delete the contact." />
		
		<!--- pass the contact bean to the DAO's delete method --->
		<cfset results = variables.contactDAO.delete(contact) />
		
		<!--- put the message from the results struct in the event object so we can display it
				to the user at the end of the next event --->
		<cfset arguments.event.setArg("message", results.message) />
		
		<!--- announce the next event --->
		<cfset announceEvent("contactDeleted", arguments.event.getArgs()) />
	</cffunction>
</cfcomponent>